@@ -26,7 +26,7 @@ def fa2tree(args):
     aln_file  = '%s/%s.aln'     % (seq_file_path, seq_file_basename)
     tree_file = '%s/%s.newick'  % (seq_file_path, seq_file_basename)
 
-    cmd_mafft    = 'mafft --quiet --maxiterate 1000 --thread %s --globalpair %s > %s' % (thread_num, seq_file, aln_file)
+    cmd_mafft    = 'mafft --quiet --maxiterate 1000 --adjustdirection --thread %s --globalpair %s > %s' % (thread_num, seq_file, aln_file)
     print(cmd_mafft)
     os.system(cmd_mafft)
 
